/** ============================================================================
MIT License

Copyright (c) 2023-2024 Institute for Automotive Engineering (ika), RWTH Aachen University

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
============================================================================= */

/** Auto-generated by https://github.com/ika-rwth-aachen/etsi_its_messages -----
python3 \
  utils/codegen/codegen-py/asn1ToConversionHeader.py \
  asn1/raw/is_ts103301/MAPEM-PDU-Descriptions.asn \
  asn1/raw/is_ts103301/cdd/ITS-Container.asn \
  asn1/raw/is_ts103301/iso-patched/ISO24534-3_ElectronicRegistrationIdentificationVehicleDataModule-patched.asn \
  asn1/raw/is_ts103301/build/asn1/ISO-TS-19091-addgrp-C-2018-patched.asn \
  asn1/patched/is_ts103301/build/asn1/ISO14816_AVIAEINumberingAndDataStructures.asn \
  -t \
  mapem_ts \
  -o \
  etsi_its_conversion/etsi_its_mapem_ts_conversion/include/etsi_its_mapem_ts_conversion
----------------------------------------------------------------------------- */

/** ASN.1 Definition -----------------------------------------------------------
ComputedLane ::= SEQUENCE {
  referenceLaneId    LaneID,
  offsetXaxis        CHOICE {
                        small   DrivenLineOffsetSm, 
                        large   DrivenLineOffsetLg
                        },  
  offsetYaxis        CHOICE {
                        small   DrivenLineOffsetSm, 
                        large   DrivenLineOffsetLg
                        },  
  rotateXY           Angle OPTIONAL, 
  scaleXaxis         Scale-B12 OPTIONAL, 
  scaleYaxis         Scale-B12 OPTIONAL, 
  regional  SEQUENCE (SIZE(1..4)) OF 
            RegionalExtension {{Reg-ComputedLane}} OPTIONAL,
----------------------------------------------------------------------------- */

#pragma once

#include <etsi_its_mapem_ts_coding/mapem_ts_ComputedLane.h>
#include <etsi_its_mapem_ts_conversion/convertAngle.h>
#include <etsi_its_mapem_ts_conversion/convertDrivenLineOffsetLg.h>
#include <etsi_its_mapem_ts_conversion/convertDrivenLineOffsetSm.h>
#include <etsi_its_mapem_ts_conversion/convertLaneID.h>
#include <etsi_its_mapem_ts_conversion/convertScaleB12.h>
#ifdef ROS1
#include <etsi_its_mapem_ts_msgs/ComputedLane.h>
namespace mapem_ts_msgs = etsi_its_mapem_ts_msgs;
#else
#include <etsi_its_mapem_ts_msgs/msg/computed_lane.hpp>
namespace mapem_ts_msgs = etsi_its_mapem_ts_msgs::msg;
#endif


namespace etsi_its_mapem_ts_conversion {

void toRos_ComputedLane(const mapem_ts_ComputedLane_t& in, mapem_ts_msgs::ComputedLane& out) {
  toRos_LaneID(in.referenceLaneId, out.reference_lane_id);
  if (in.offsetXaxis.present == mapem_ts_ComputedLane__offsetXaxis_PR::mapem_ts_ComputedLane__offsetXaxis_PR_small) {
    toRos_DrivenLineOffsetSm(in.offsetXaxis.choice.small, out.offset_xaxis_small);
    out.offset_xaxis_choice = mapem_ts_msgs::ComputedLane::CHOICE_OFFSET_XAXIS_SMALL;
  }
  if (in.offsetXaxis.present == mapem_ts_ComputedLane__offsetXaxis_PR::mapem_ts_ComputedLane__offsetXaxis_PR_large) {
    toRos_DrivenLineOffsetLg(in.offsetXaxis.choice.large, out.offset_xaxis_large);
    out.offset_xaxis_choice = mapem_ts_msgs::ComputedLane::CHOICE_OFFSET_XAXIS_LARGE;
  }
  if (in.offsetYaxis.present == mapem_ts_ComputedLane__offsetYaxis_PR::mapem_ts_ComputedLane__offsetYaxis_PR_small) {
    toRos_DrivenLineOffsetSm(in.offsetYaxis.choice.small, out.offset_yaxis_small);
    out.offset_yaxis_choice = mapem_ts_msgs::ComputedLane::CHOICE_OFFSET_YAXIS_SMALL;
  }
  if (in.offsetYaxis.present == mapem_ts_ComputedLane__offsetYaxis_PR::mapem_ts_ComputedLane__offsetYaxis_PR_large) {
    toRos_DrivenLineOffsetLg(in.offsetYaxis.choice.large, out.offset_yaxis_large);
    out.offset_yaxis_choice = mapem_ts_msgs::ComputedLane::CHOICE_OFFSET_YAXIS_LARGE;
  }
  if (in.rotateXY) {
    toRos_Angle(*in.rotateXY, out.rotate_xy);
    out.rotate_xy_is_present = true;
  }
  if (in.scaleXaxis) {
    toRos_ScaleB12(*in.scaleXaxis, out.scale_xaxis);
    out.scale_xaxis_is_present = true;
  }
  if (in.scaleYaxis) {
    toRos_ScaleB12(*in.scaleYaxis, out.scale_yaxis);
    out.scale_yaxis_is_present = true;
  }
}

void toStruct_ComputedLane(const mapem_ts_msgs::ComputedLane& in, mapem_ts_ComputedLane_t& out) {
  memset(&out, 0, sizeof(mapem_ts_ComputedLane_t));
  toStruct_LaneID(in.reference_lane_id, out.referenceLaneId);
  if (in.offset_xaxis_choice == mapem_ts_msgs::ComputedLane::CHOICE_OFFSET_XAXIS_SMALL) {
    toStruct_DrivenLineOffsetSm(in.offset_xaxis_small, out.offsetXaxis.choice.small);
    out.offsetXaxis.present = mapem_ts_ComputedLane__offsetXaxis_PR::mapem_ts_ComputedLane__offsetXaxis_PR_small;
  }
  if (in.offset_xaxis_choice == mapem_ts_msgs::ComputedLane::CHOICE_OFFSET_XAXIS_LARGE) {
    toStruct_DrivenLineOffsetLg(in.offset_xaxis_large, out.offsetXaxis.choice.large);
    out.offsetXaxis.present = mapem_ts_ComputedLane__offsetXaxis_PR::mapem_ts_ComputedLane__offsetXaxis_PR_large;
  }
  if (in.offset_yaxis_choice == mapem_ts_msgs::ComputedLane::CHOICE_OFFSET_YAXIS_SMALL) {
    toStruct_DrivenLineOffsetSm(in.offset_yaxis_small, out.offsetYaxis.choice.small);
    out.offsetYaxis.present = mapem_ts_ComputedLane__offsetYaxis_PR::mapem_ts_ComputedLane__offsetYaxis_PR_small;
  }
  if (in.offset_yaxis_choice == mapem_ts_msgs::ComputedLane::CHOICE_OFFSET_YAXIS_LARGE) {
    toStruct_DrivenLineOffsetLg(in.offset_yaxis_large, out.offsetYaxis.choice.large);
    out.offsetYaxis.present = mapem_ts_ComputedLane__offsetYaxis_PR::mapem_ts_ComputedLane__offsetYaxis_PR_large;
  }
  if (in.rotate_xy_is_present) {
    out.rotateXY = (mapem_ts_Angle_t*) calloc(1, sizeof(mapem_ts_Angle_t));
    toStruct_Angle(in.rotate_xy, *out.rotateXY);
  }
  if (in.scale_xaxis_is_present) {
    out.scaleXaxis = (mapem_ts_Scale_B12_t*) calloc(1, sizeof(mapem_ts_Scale_B12_t));
    toStruct_ScaleB12(in.scale_xaxis, *out.scaleXaxis);
  }
  if (in.scale_yaxis_is_present) {
    out.scaleYaxis = (mapem_ts_Scale_B12_t*) calloc(1, sizeof(mapem_ts_Scale_B12_t));
    toStruct_ScaleB12(in.scale_yaxis, *out.scaleYaxis);
  }
}

}
